////////////////////////////////////////////////////////////////////////////////
//                                 
//                        (c)  , 1999-2000
//    http://mazurkin.virtualave.net, mazurkin@mailru.com, mazurkin@chat.ru
// _____________________________________________________________________________
//                                
////////////////////////////////////////////////////////////////////////////////

unit LssWindowSaver;

interface

uses
  Windows, Messages, WinLite;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
type
  TSaverParameters = record
    PartsCount : Integer;
  end;

type
  TWindowSaver = class(TLiteWindow)
  private
    FIsPreview  : Boolean;
    FMouseTicks : Integer;
    FRedrawTimer: Integer;
    FParameters : TSaverParameters;
  protected
    procedure   CreateWindowParams(var WindowParams: TWindowParams); override;
    procedure   WindowProcedure(var Msg: TMessage); override;
  protected
    procedure   OnSysCommand(var Msg: TWMSysCommand); message WM_SYSCOMMAND;
    procedure   OnCreate(var Msg: TWMCreate); message WM_CREATE;
    procedure   OnDestroy(var Msg: TWMDestroy); message WM_DESTROY;
    procedure   OnEraseBkgnd(var Msg: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure   OnPaint(var Msg: TWMPaint); message WM_PAINT;
    procedure   OnTimer(var Msg: TWMTimer); message WM_TIMER;
    procedure   OnSize(var Msg: TWMSize); message WM_SIZE;
  protected
    procedure   TerminateScreenSaver;
    function    IsPasswordNeeded: Boolean;
  public
    procedure   InitSaver(const Parameters: TSaverParameters);
    constructor CreateSaver(AWndParent: THandle; Preview: Boolean);
    class procedure SaverRun;
    class procedure SaverPreview(ParentWnd: THandle);
  end;

implementation

uses
  AnlDialogSetup;

////////////////////////////////////////////////////////////////////////////////
// TWindowSaver
// _____________________________________________________________________________
//   
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
class procedure TWindowSaver.SaverPreview(ParentWnd: THandle);
var
  Saver : TWindowSaver;
  Msg   : TMsg;
begin
  //  
  Saver := TWindowSaver.CreateSaver(ParentWnd, True);
  //   
  while GetMessage(Msg, 0, 0, 0) do
  begin
    TranslateMessage(Msg);
    DispatchMessage(Msg);
  end;
  //  
  Saver.Free;
end;

////////////////////////////////////////////////////////////////////////////////
//     
////////////////////////////////////////////////////////////////////////////////
class procedure TWindowSaver.SaverRun;
var
  Saver   : TWindowSaver;
  Msg     : TMsg;
begin
  //  
  Saver := TWindowSaver.CreateSaver(GetDesktopWindow, False);
  //    
  SystemParametersInfo(SPI_SCREENSAVERRUNNING, 1, nil, 0);
  //  
  ShowCursor(False);
  //   
  while GetMessage(Msg, 0, 0, 0) do
  begin
    TranslateMessage(Msg);
    DispatchMessage(Msg);
  end;
  //  
  ShowCursor(True);
  //     
  SystemParametersInfo(SPI_SCREENSAVERRUNNING, 0, nil, 0);
  //  
  Saver.Free;
end;

////////////////////////////////////////////////////////////////////////////////
// TWindowSaver
// _____________________________________________________________________________
//  / 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// 
////////////////////////////////////////////////////////////////////////////////
constructor TWindowSaver.CreateSaver(AWndParent: THandle; Preview: Boolean);
begin
  //   
  FIsPreview  := Preview;
  //    
  InitSaver(LoadParameters);
  //  
  inherited Create(AWndParent);
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.CreateWindowParams(var WindowParams: TWindowParams);
var
  R : TRect;
begin
  inherited;
  //   
  WindowParams.Caption := '';
  WindowParams.WindowClass.hbrBackground := 1+COLOR_BTNFACE;
  //     ,     
  if FIsPreview then
  begin
    //     
    WindowParams.WindowClass.style := CS_HREDRAW or CS_VREDRAW or CS_DBLCLKS;
    WindowParams.Style   := WS_CHILD or WS_DISABLED or WS_VISIBLE;
    //    
    GetWindowRect(WindowParams.WndParent, R);
    //     
    WindowParams.X      := 0;
    WindowParams.Y      := 0;
    WindowParams.Width  := R.Right  - R.Left;
    WindowParams.Height := R.Bottom - R.Top;
  end else
  begin
    //    
    WindowParams.WindowClass.style := CS_HREDRAW or CS_VREDRAW or CS_DBLCLKS;
    WindowParams.Style   := WS_POPUP or WS_VISIBLE;
    WindowParams.ExStyle := WS_EX_TOPMOST;
    //     
    WindowParams.X      := 0;
    WindowParams.Y      := 0;
    WindowParams.Width  := GetSystemMetrics(SM_CXSCREEN);
    WindowParams.Height := GetSystemMetrics(SM_CYSCREEN);
  end;
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.OnCreate(var Msg: TWMCreate);
begin
  inherited;
  //   
  FRedrawTimer := SetTimer(WndHandle, 2, 100, nil);
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.OnDestroy(var Msg: TWMDestroy);
begin
  inherited;
  //   
  KillTimer(WndHandle, FRedrawTimer);
  //   
  PostQuitMessage(0);
end;

////////////////////////////////////////////////////////////////////////////////
// TWindowSaver
// _____________________________________________________________________________
//  
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.WindowProcedure(var Msg: TMessage);
begin
  //     ,        
  if not FIsPreview then
    //    
    case Msg.Msg of
      //     
      WM_KEYDOWN, WM_SYSKEYDOWN,
      WM_LBUTTONDOWN,   WM_MBUTTONDOWN,   WM_RBUTTONDOWN,
      WM_LBUTTONDBLCLK, WM_MBUTTONDBLCLK, WM_RBUTTONDBLCLK :
          TerminateScreenSaver;
      //     
      WM_MOUSEMOVE:
          //     ?
          if FMouseTicks > 2 then TerminateScreenSaver
                             else Inc(FMouseTicks);
    end;
  //      
  inherited;
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.OnSysCommand(var Msg: TWMSysCommand);
begin
  //   ,    
  case Msg.CmdType of
    SC_SCREENSAVE : Msg.Result := 0;
    SC_CLOSE      : Msg.Result := 0;
    else            inherited;
  end;  
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.OnTimer(var Msg: TWMTimer);
var
  KillMsg : TMsg;
begin
  //     ,    
  if FMouseTicks > 0 then Dec(FMouseTicks);
  //  
  InvalidateRect(WndHandle, nil, False);
  //     
  repeat until not PeekMessage(KillMsg, WndHandle, WM_TIMER, WM_TIMER, PM_REMOVE);
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.OnSize(var Msg: TWMSize);
begin
  //      
  InitSaver(FParameters);
end;

////////////////////////////////////////////////////////////////////////////////
// TWindowSaver
// _____________________________________________________________________________
// 
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//    
////////////////////////////////////////////////////////////////////////////////
function TWindowSaver.IsPasswordNeeded: Boolean;
var
  RK    : HKEY;
  Size  : DWord;
  Value : DWord;
begin
  //      
  RegOpenKeyEx(HKEY_CURRENT_USER, 'Control Panel\desktop', 0, KEY_READ, RK);
  //  
  Size := 4;
  RegQueryValueEx(RK, 'ScreenSaveUsePassword', nil, nil, @Value, @Size);
  //  
  RegCloseKey(RK);
  // 
  Result := Value <> 0;
end;

////////////////////////////////////////////////////////////////////////////////
//   
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.TerminateScreenSaver;
type
  TCheckPasswordFunction = function (Parent: THandle): Boolean; stdcall;
var
  CheckLib  : THandle;
  CheckProc : TCheckPasswordFunction;
  CheckPath : string;
begin
  //      
  FMouseTicks := 0;
  //  -     ?
  if IsPasswordNeeded then
  begin
    //   
    SetLength(CheckPath, MAX_PATH);
    SetLength(CheckPath, GetSystemDirectory(PChar(CheckPath), MAX_PATH));
    //     
    if CheckPath[Length(CheckPath)] = '\' then CheckPath := CheckPath + 'PASSWORD.CPL'
                                          else CheckPath := CheckPath + '\PASSWORD.CPL';
    //   
    CheckLib := LoadLibrary(PChar(CheckPath));
    //     -  
    if CheckLib <> 0 then
    begin
      //     
      @CheckProc := GetProcAddress(CheckLib, 'VerifyScreenSavePwd');
      //   ?
      if @CheckProc <> nil then
      begin
        //   
        ShowCursor(True);
        //  
        if CheckProc(WndHandle) then DestroyWindow(WndHandle)
                                else Exit;
        //  
        ShowCursor(False);
      end
      else
        DestroyWindow(WndHandle);
      //  
      FreeLibrary(CheckLib);
    end
    else
      DestroyWindow(WndHandle);
  end
  else
    DestroyWindow(WndHandle);
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.InitSaver(const Parameters: TSaverParameters);
begin
  //  
  FParameters := Parameters;
end;

////////////////////////////////////////////////////////////////////////////////
// TWindowSaver
// _____________________________________________________________________________
//  
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.OnEraseBkgnd(var Msg: TWMEraseBkgnd);
begin
  //    ,   
  Msg.Result := 1;
end;

////////////////////////////////////////////////////////////////////////////////
//  
////////////////////////////////////////////////////////////////////////////////
procedure TWindowSaver.OnPaint(var Msg: TWMPaint);
var
  PS : TPaintStruct;
  R  : TRect;
begin
  //  
  BeginPaint(WndHandle, PS);
  //   
  GetWindowRect(WndHandle, R);
  //     
  OffsetRect(R, -R.Left, -R.Top);
  //    
  FillRect(PS.hdc, R, GetStockObject(BLACK_BRUSH));
  //  
  EndPaint(WndHandle, PS);
end;

end.
